class Swap {
    constructor(name, callback) {
        this.name = name;

        this.thenCallback = [];

        this.attached = [];

        this.parentElement = document.querySelector("[data-swap='" + name + "']");

        if (! this.parentElement ) {
            console.error("You must provide a valid HTMLElement to swap it. Impossible to query `" + name + "` element.");
        }

        this.actionCallback = callback;

        this.swap();
    }

    swapHtml(html) {
        this.parentElement.innerHTML = ( new DOMParser() )
            .parseFromString(html, "text/html")
            .querySelector('[data-swap="' + this.name + '"]')
            .innerHTML;
            
        this.parentElement.classList.remove("swapping");

        this.swap();

        this.attached.forEach(swapObj => {
            swapObj.swapHtml(html);
        });
    }

    swapUrl(url, callback) {
        this.parentElement.classList.add("swapping");
        
        fetch(url).then( response => response.text() ).then(html => {
            this.swapHtml(html);

            if (callback) {
                callback.call(this,  this);
            }
        });

        return this;
    }

    swapForm(element) {
        if (element instanceof HTMLElement) {
            this.swapSingleForm(element);
        }
        else if (element instanceof HTMLCollection || element instanceof NodeList) {
            element.forEach(ele => this.swapSingleForm(ele));
        }

        return this;
    }
    
    swapSingleForm(element) {
        element.addEventListener("submit", e => {
            e.preventDefault();
            e.stopImmediatePropagation();

            let body = new FormData(element);

            if (document.activeElement && document.activeElement.hasAttribute('name')) {
                body.append(document.activeElement.getAttribute('name'), document.activeElement.value);
            }                

            let url = element.getAttribute("action") ? element.getAttribute("action") : location.href,
                method = element.getAttribute("method") ? element.getAttribute("method") : "get";

            fetch(url, {
                body: body,
                method: method
            }).then( response => response.text() ).then( 
                html => {
                    this.swapHtml(html);
                }
            );
        });

        return this;
    }

    swap() {
        this.actionCallback && this.actionCallback(this);
    }

    then(callback) {
        this.thenCallback.push[callback];

        return this;
    }

    attach(element) {
        (Symbol.iterator in Object(element) ? element : [ element ]).forEach(e => {
            this.attached.push(e);
        });

        return this;
    }

    get form() {
        return this._form;
    }

    set form(value) {
        return this._form = value;
    }

    get actionCallback() {
        return this._actionCallback;
    }

    set actionCallback(value) {
        return this._actionCallback = value;
    }

    get parentElement() {
        return this._parentElement;
    }

    set parentElement(value) {
        return this._parentElement = value;
    }

    get name() {
        return this._name;
    }

    set name(value) {
        return this._name = value;
    }

    get thenCallback() {
        return this._thenCallback;
    }

    set thenCallback(value) {
        return this._thenCallback = value;
    }

    get attached() {
        return this._attached;
    }

    set attached(value) {
        return this._attached = value;
    }
}